/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_FUSIONSERVICE_FUSIONSERVICE_PROXY_H
#define JAC_AP_FUSIONSERVICE_FUSIONSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/fusionservice/fusionservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace fusionservice {
namespace proxy {
namespace events {
    using ObjectEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::TrafficParticipantStruct>;
    using Fusion2VizRulebasedResultEvent = ara::com::internal::proxy::event::EventAdapter<::PerceptionBoundingBoxArray>;
    static constexpr ara::com::internal::EntityId FusionServiceObjectEventId = 35544U; //ObjectEvent_event_hash
    static constexpr ara::com::internal::EntityId FusionServiceFusion2VizRulebasedResultEventId = 55454U; //Fusion2VizRulebasedResultEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class FusionServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              ObjectEvent(proxyAdapter->GetProxy(), events::FusionServiceObjectEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              Fusion2VizRulebasedResultEvent(proxyAdapter->GetProxy(), events::FusionServiceFusion2VizRulebasedResultEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::ObjectEvent GetObjectEvent() noexcept { return std::move(ObjectEvent); }
        events::Fusion2VizRulebasedResultEvent GetFusion2VizRulebasedResultEvent() noexcept { return std::move(Fusion2VizRulebasedResultEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::ObjectEvent ObjectEvent;
        events::Fusion2VizRulebasedResultEvent Fusion2VizRulebasedResultEvent;
    };

    virtual ~FusionServiceProxy()
    {
        ObjectEvent.UnsetReceiveHandler();
        ObjectEvent.Unsubscribe();
        Fusion2VizRulebasedResultEvent.UnsetReceiveHandler();
        Fusion2VizRulebasedResultEvent.Unsubscribe();

    }

    explicit FusionServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::fusionservice::FusionService::ServiceIdentifier, handle)),
          ObjectEvent(proxyAdapter->GetProxy(), events::FusionServiceObjectEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          Fusion2VizRulebasedResultEvent(proxyAdapter->GetProxy(), events::FusionServiceFusion2VizRulebasedResultEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    FusionServiceProxy(const FusionServiceProxy&) = delete;
    FusionServiceProxy& operator=(const FusionServiceProxy&) = delete;

    FusionServiceProxy(FusionServiceProxy&&) = default;
    FusionServiceProxy& operator=(FusionServiceProxy&&) = default;
    FusionServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          ObjectEvent(token.GetObjectEvent()),
          Fusion2VizRulebasedResultEvent(token.GetFusion2VizRulebasedResultEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::fusionservice::FusionService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<FusionServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::fusionservice::FusionService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::fusionservice::FusionService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::fusionservice::FusionService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::fusionservice::FusionService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::ObjectEvent ObjectEvent;
    events::Fusion2VizRulebasedResultEvent Fusion2VizRulebasedResultEvent;
};
} // namespace proxy
} // namespace fusionservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_FUSIONSERVICE_FUSIONSERVICE_PROXY_H
